@@ -1,6 +1,9 @@
 import asyncio
-import json
+import aiohttp
 import subprocess
+import warnings
+from contextlib import suppress
+import json
 
 from distributed.deploy import Cluster
 from distributed.core import rpc, Status
@@ -92,6 +95,7 @@ class HelmCluster(Cluster):
     ):
         self.release_name = release_name
         self.namespace = namespace or namespace_default()
+        self.name = self.release_name + "." + self.namespace
         check_dependency("helm")
         check_dependency("kubectl")
         status = subprocess.run(
@@ -266,11 +270,46 @@ class HelmCluster(Cluster):
     def adapt(self, *args, **kwargs):
         """Turn on adaptivity (Not recommended)."""
         raise NotImplementedError(
-            "It is not recommended to run ``HelmCluster`` in adaptive mode."
-            "When scaling down workers the decision on which worker to remove is left to Kubernetes, which"
-            "will not necessarily remove the same worker that Dask would choose. This may result in lost futures and"
+            "It is not recommended to run ``HelmCluster`` in adaptive mode. "
+            "When scaling down workers the decision on which worker to remove is left to Kubernetes, which "
+            "will not necessarily remove the same worker that Dask would choose. This may result in lost futures and "
             "recalculation. It is recommended to manage scaling yourself with the ``HelmCluster.scale`` method."
         )
 
     async def _adapt(self, *args, **kwargs):
         return super().adapt(*args, **kwargs)
+
+    async def _close(self, *args, **kwargs):
+        """Close the cluster."""
+        warnings.warn(
+            "It is not possible to close a HelmCluster object. \n"
+            "Please delete the cluster via the helm CLI: \n\n"
+            f"  $ helm delete --namespace {self.namespace} {self.release_name}"
+        )
+
+    @classmethod
+    def from_name(cls, name):
+        release_name, namespace = name.split(".")
+        return cls(release_name=release_name, namespace=namespace)
+
+
+async def discover(
+    auth=ClusterAuth.DEFAULT,
+    namespace=None,
+):
+    await ClusterAuth.load_first(auth)
+    async with kubernetes.client.api_client.ApiClient() as api:
+        core_api = kubernetes.client.CoreV1Api(api)
+        namespace = namespace or namespace_default()
+        try:
+            pods = await core_api.list_pod_for_all_namespaces(
+                label_selector="app=dask,component=scheduler",
+            )
+            for pod in pods.items:
+                with suppress(KeyError):
+                    yield (
+                        pod.metadata.labels["release"] + "." + pod.metadata.namespace,
+                        HelmCluster,
+                    )
+        except aiohttp.client_exceptions.ClientConnectorError:
+            warnings.warn("Unable to connect to Kubernetes cluster")
